/*
 * This simulation takes events from simulation lev. Sometime can receive
 * anti-message and must rolled back events.
 */
import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.unsecure.*;

public class QueueingNetworksExample_5 {
    
    
    public static void main(String[] args) {

        DistributedSimulation   simulation = null;
        
        QueueWithServer         queue1 = null;
        Server                  server1 = null;        
        NetPoint                net_point1 = null;
        
        double                  mu1 = 1.0;
        double                  p1 = 0.7;
        
        ClientMainThread        client;

        ClientData              data;        
         
        try {
            System.out.println("Initializing the simulation..");
            
            simulation = new DistributedSimulation("Queueing Networks Simulation");
                        
            data = new ClientData(simulation);
            
            queue1 = new QueueWithServer("Queue 1", simulation, null);
            
            client = new ClientMainThread(data);
            
            server1 = new Server("Server 1", simulation, mu1, p1, queue1, null, data);
            
            net_point1 = new NetPoint("Net Point 1", simulation, queue1, data);
            
   /*
    *  We must set the servers now because they didn't exist at the time the queues were created.
    */
            
            queue1.setServer(server1);
            
            // set simulation name 
            data.setSimulationName("opice");
            
            // set receiver simulation name 
            // if  there no receiver name you must set name as "NO"
            data.setReceiverName("no");
            // when is netpoint present you must set simulation end time
            net_point1.setSimulationEndTime(1000.0);
            
            data.addServerAndQueueRef(server1, queue1);            
            // simulation status - depend
            data.setSimulationStatus(2);
            data.setOutputStatus(2);
            
            client.start();
            while (data.comm_ready == false){
                try { /* take a nap */
                    Thread.currentThread().sleep(100);
                }
                catch (InterruptedException E){
                    System.out.println("Get up !!");
                } // end catch
            };
            
            net_point1.activate(0.0);
            
            simulation.message("Activating the generators...");
                        
            simulation.message("Running the simulation, please wait.");
            
            while ((simulation.getCurrentTime() < 1000.0) && (simulation.step() == true)){
                
            }
            

            data.putToOutputQueue("sim_over");
            
            while(data.order_so_ack == false) {
                data.thread_sleep(200);
            }
            
            data.putToOutputQueue("cexit");
            
   /*
    *  Now, some boring numbers.
    */
            simulation.message("Simulation interrupted at time " + simulation.getCurrentTime());
            simulation.message("Queues' statistics:");
            simulation.message("Queue 1: Lw = " + queue1.getLw() + ", Tw = " + queue1.getTw() + ", Tw all = " + queue1.getTwForAllLinks());
            simulation.message("Servers' statistics:");
            simulation.message("Server 1: Counter = " + server1.getCounter() + ", sum of Tq (for transactions thrown away by this server) = " + server1.getTransTq());
            simulation.message("Server 1: Counter Distributed = " + data.output_queue.size());
            simulation.message("Number of components in Simulation = " +  data.simData.allSimItem.size());    
            simulation.message("Number of undo operation in Simulation = " + simulation.getNumberOfUndo());
            simulation.message("Number of roolback operation in Simulation = " + simulation.getNumberOfRollBack());
           
            try {
                client.join();
            }
            catch (InterruptedException e){
                System.err.println("Exception: " + e.getMessage());
                System.exit(1);
            }
            data = null;
            
        } // try
        catch (JSimException e) {
            e.printStackTrace();
            e.printComment(System.err);
        }
        
  /*
   *  We must shutdown the simulation.
   */
        finally {
            simulation.shutdown();
        }
    } // main
} // class QueueingNetworksExample

