/*
 * This simulation generate events for simulation kocka
 *
 * In this distributed simulation can occur deadlock situation but 
 * time lock can catch it. 
 *
 */
import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.unsecure.*;

public class QueueingNetworksExample_1 {
    
    
    public static void main(String[] args) {
        
        DistributedSimulation  simulation = null;
        
        QueueWithServer         queue1 = null;
        Server                  server1 = null;
        Generator               generator1 = null;
        
        double                  mu1 = 1.0, mu2 = 1.0;
        double                  lambda1 = 0.1, lambda2 = 0.4;
        double                  p1 = 0.5;

        ClientMainThread        client;
        ClientData              data;        
        
        try {
            System.out.println("Initializing the simulation..");
            
            simulation = new DistributedSimulation("Queueing Networks Simulation");
            
            data = new ClientData(simulation);                        
            
            queue1 = new QueueWithServer("Queue 1", simulation, null);
            
            client = new ClientMainThread(data);
            
            server1 = new Server("Server 1", simulation, mu1, p1, queue1, null, data, "kocka", 0);
            
            generator1 = new Generator("Generator 1", simulation, lambda1, queue1, data);
            
   /*
    *  We must set the servers now because they didn't exist at the time the queues were created.
    */
            
            queue1.setServer(server1);
            // set simulation name
            data.setSimulationName("pes");            
            // set receiver simulation name
            data.setReceiverName("kocka");
            // store simulations componets
            data.addServerAndQueueRef(server1, queue1);
            // set simulation status
            data.setSimulationStatus(1);
            // set simulation output status
            data.setOutputStatus(2);
            
            client.start();
            
            while (data.comm_ready == false){
                try { /* take a nap */
                    Thread.currentThread().sleep(100);
                }
                catch (InterruptedException E){
                    System.out.println("Get up !!");
                } // end catch
            };
            
            simulation.message("Activating the generators...");
            
            generator1.activate(0.0);
            
            simulation.message("Running the simulation, please wait.");
            
            while ((simulation.getCurrentTime() < 1000.0) && (simulation.step() == true)){
                
            }
            
            data.putToOutputQueue("last_time");
            data.putToOutputQueue("sim_over");
            
            while(data.order_so_ack == false) {
                data.thread_sleep(200);
            }
            
            data.putToOutputQueue("cexit");
            
   /*
    *  Now, some boring numbers.
    */
            simulation.message("Simulation interrupted at time " + simulation.getCurrentTime());
            simulation.message("Queues' statistics:");
            simulation.message("Queue 1: Lw = " + queue1.getLw() + ", Tw = " + queue1.getTw() + ", Tw all = " + queue1.getTwForAllLinks());
            simulation.message("Servers' statistics:");
            simulation.message("Server 1: Counter = " + server1.getCounter() + ", sum of Tq (for transactions thrown away by this server) = " + server1.getTransTq());
            simulation.message("Server 1: Counter Distributed = " + data.output_queue.size());
            simulation.message("Number of components in Simulation = " +  data.simData.allSimItem.size());

            try {
                client.join();
            }
            catch (InterruptedException e){
                System.err.println("Exception: " + e.getMessage());
                System.exit(1);
            }
            data = null;
            
        } // try
        catch (JSimException e) {
            e.printStackTrace();
            e.printComment(System.err);
        }
        
  /*
   *  We must shutdown the simulation.
   */
        finally {
            simulation.shutdown();
        }
    } // main
} // class QueueingNetworksExample

