/*
/*
 * This simulation takes events from simulation kocka and sometime can receive
 * anti-message and must rolled back events. and must send anti-message to simulation 
 * kocka.
 */
import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.unsecure.*;

public class QueueingNetworksExample_3 {
    public static void main(String[] args) {
        
        DistributedSimulation           simulation = null;
        
        QueueWithServer       queue1 = null;
        QueueWithServer       queue2 = null;
        //Generator             generator1 = null;
        Server                server1 = null;
        Server                server2 = null;        
        NetPoint              net_point1 = null;
        
        double                mu1 = 1.0, mu2 = 1.0;
        double                lambda1 = 0.4, lambda2 = 0.4;
        double                p1 = 0.5, p2 = 0.5;
        
        ClientMainThread      client;
        ClientData            data;        
        
        try
        
        {
            System.out.println("Initializing the simulation..");
            
            simulation = new DistributedSimulation("Queueing Networks Simulation");
            
            data = new ClientData(simulation);                        
            
            queue1 = new QueueWithServer("Queue 1", simulation, null);
            queue2 = new QueueWithServer("Queue 2", simulation, null);                

            // this server is "classic" server no out point
            server1 = new Server("Server 1", simulation, mu1, p1, queue1, queue2, data);
            // this server is out point to distrib simulation there you store events to output queue
            server2 = new Server("Server 2", simulation, mu1, p1, queue2, null, data, "kocka", 0);
            
            net_point1 = new NetPoint("Net Point 1", simulation, queue1, data);
            
            client = new ClientMainThread(data);
            
            queue1.setServer(server1);
            queue2.setServer(server2);
            
            // set simulation name
            data.setSimulationName("lev");            
            // set receiver simulation name
            data.setReceiverName("kocka");
            
            // set simulation end time
            net_point1.setSimulationEndTime(1000.0);
            
            // set sender simulation name
            data.addSenderName("kocka");
            
            // store componets referencis
            data.addServerAndQueueRef(server1, queue1);    
            data.addServerAndQueueRef(server2, queue2);    
            // set simulation status
            data.setSimulationStatus(2);
            data.setOutputStatus(2);
            
            client.start();
            
            while (data.comm_ready == false){
                try { /* take a nap */
                    Thread.currentThread().sleep(100);
                }
                catch (InterruptedException E){
                    System.out.println("Get up !!");
                } // end catch
            };
            
            simulation.message("Activating the generators...");
            
            net_point1.activate(0.0);
            
            while ((simulation.getCurrentTime() < 1000.0) && (simulation.step() == true)){
            }

            data.putToOutputQueue("last_time");
            data.putToOutputQueue("sim_over");
                        
            while(data.order_so_ack == false) {
                data.thread_sleep(200);
            }
 
            data.putToOutputQueue("cexit");

   /*
    *  Now, some boring numbers.
    */
            
            simulation.message("Simulation interrupted at time " + simulation.getCurrentTime());
            simulation.message("Queues' statistics:");
            simulation.message("Queue 1: Lw = " + queue1.getLw() + ", Tw = " + queue1.getTw() + ", Tw all = " + queue1.getTwForAllLinks());
            simulation.message("Queue 2: Lw = " + queue2.getLw() + ", Tw = " + queue2.getTw() + ", Tw all = " + queue1.getTwForAllLinks());
            simulation.message("Servers' statistics:");
            simulation.message("Server 1: Counter = " + server1.getCounter() + ", sum of Tq (for transactions thrown away by this server) = " + server1.getTransTq());
            simulation.message("Server 2: Counter = " + server2.getCounter() + ", sum of Tq (for transactions thrown away by this server) = " + server2.getTransTq());
            simulation.message("Number of components in Simulation = " +  data.simData.allSimItem.size());
            simulation.message("Number of undo operation in Simulation = " + simulation.getNumberOfUndo());
            simulation.message("Number of roolback operation in Simulation = " + simulation.getNumberOfRollBack());
            
            try {
                client.join();
            }
            catch (InterruptedException e){
                System.err.println("Exception: " + e.getMessage());
                System.exit(1);
            }
            
        } // try
        catch (JSimException e) {
            e.printStackTrace();
            e.printComment(System.err);
        }
        
  /*
   *  We must shutdown the simulation.
   */
        finally {
            simulation.shutdown();
        }
    } // main
        
} // class QueueingNetworksExample

