/*
 *  A simple generator. Creates new transactions and puts them into a queue.
 *  Look into the documentation for additional information.
 *
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.unsecure.*;

public class DistributedSimulation extends UnsecureSimulation {
    
    private double                 newTime = 0.0;
    private int                    numberOfUndo = 0;
    private int                    numberOfRollBack = 0;
    
    public DistributedSimulation(String name) throws JSimInvalidParametersException {
        super(name, JSimSimulation.MODE_TXT, 0.0);
    } // constructor
    
    
    public void operationUndo(double ParAbsTime, ClientData data){
        int i;
        SimServerItem   t_item_1;
        GeneratorRef    t_item_2;
        double delta = 0.2;
        
        setUndoComing(true);
        newTime = ParAbsTime;
        setTime(newTime);
        numberOfUndo++;
        
        /*
         *  Set new item to calendar for Server Item
         */
        //message("----------------------");
        
         /*
          *  Set new item to calendar for Generator Item
          */
        
        for(i = 0; i < data.ref_field.size(); i++) {
            
            t_item_2 = ((GeneratorRef) data.ref_field.get(i));
            //System.out.println("0"+t_item_2.Generator.getName() + t_item_2.Generator.getStateAsString());
            
            try {
                if (t_item_2.Generator.getState() == 1) {
                   // System.out.println("1"+t_item_2.Generator.getName() + t_item_2.Generator.getStateAsString() + "delta:"+ delta);
                    t_item_2.Generator.activate(newTime + 0.1);
                }
                if (t_item_2.Generator.getState() == 2) {
                    //System.out.println("2"+t_item_2.Generator.getName() + t_item_2.Generator.getStateAsString());
                    t_item_2.Generator.cancel();
                    //System.out.println("2"+t_item_2.Generator.getName() + t_item_2.Generator.getStateAsString());
                    t_item_2.Generator.activate(newTime + 0.1);
                }
                //System.out.println("3" + t_item_2.Generator.getName() + t_item_2.Generator.getStateAsString());
            }
            catch (JSimInvalidParametersException e) {
                System.out.println("Exception :" + t_item_2.Generator.getStateAsString());
            }
            catch (JSimSecurityException e) {
                System.out.println("Exception :" + t_item_2.Generator.getStateAsString());
            }
        }
        
        /*
         *  Set new item to calendar for Server Item
         */
        //message("Pocet serveru:" + data.simData.allSimItem.size());
        for(i = 0; i < data.simData.allSimItem.size(); i++) {
            
            t_item_1 = ((SimServerItem) data.simData.allSimItem.get(i));
            //System.out.println("0" + t_item_1.serverName + t_item_1.serverRef.getStateAsString());
            delta = delta * (i + 1);
            try {
               // message(" count of events in calender :" + this.deleteEntriesInCalendar(t_item_1.serverRef, true));
                
                if (t_item_1.serverRef.getState() == 1) {
                   // System.out.println("1." + t_item_1.serverName + t_item_1.serverRef.getStateAsString() + "delta:"+ delta);
                    t_item_1.serverRef.activate(newTime + delta);
                }
                if (t_item_1.serverRef.getState() == 2) {
                   // System.out.println("2." + t_item_1.serverName + t_item_1.serverRef.getStateAsString());
                    t_item_1.serverRef.cancel();
                   // System.out.println("2." + t_item_1.serverName + t_item_1.serverRef.getStateAsString());
                    t_item_1.serverRef.activate(newTime + delta);
                }
                // restore all servers item
                ((Server) t_item_1.serverRef).restore(t_item_1.serverRef);
               // System.out.println("3" + t_item_1.serverName + t_item_1.serverRef.getStateAsString());
            }
            catch (JSimInvalidParametersException e) {
                System.out.println("Exception :" + t_item_1.serverRef.getStateAsString());
            }
            catch (JSimSecurityException e) {
                System.out.println("Exception :" + t_item_1.serverRef.getStateAsString());
            }
        }
        
        return;
    }
    
    public void operationRollback(double ParAntiMessageTime, ClientData data){
        double          creationTime = 0.0;
        double          newTime = 0.0;
        int             countOfItems = 0;
        boolean         sendAntiMessage = false;
        int i;
        double delta = 0.2;
        
        SimServerItem   t_item_1;
        GeneratorRef    t_item_2;
        
        data.WriteInQToFile();
        data.WriteOutQToFile();
        
        setUndoComing(true);
        
        // ziskej novy simulacni cas
        newTime = data.getPredEvent(ParAntiMessageTime);
        System.out.println("--------- R o l l  B a c  k ------------");
        System.out.println("New sim Time comput : " + newTime);
        
        // vymaze vsechny udalosti z vstupni fronty ktere maji vetsi casovou znamku
        // nez nove naplanovany cas
        countOfItems = data.eraseRolledEvent(newTime);
        
        if (newTime < this.time) {
            
            // nastavi novy simulacni cas
            setTime(newTime);
            
            numberOfRollBack++;
            
            // vymazaz vsechny vysledky vsech komponent v simulace od doby >> T_stamp(t-1)
            countOfItems = data.eraseSimItem(newTime);
            //message("Count of erased incorect processed Events in simulation :" + countOfItems);
            
            
            // musime poslat nejake anti-zpravy
            if (data.checkOutGoingMessages(newTime) == 1) {
                sendAntiMessage = true;
            }
            else {
                sendAntiMessage = false;
            }
            
            // vymaz vsechni komunikacni buffery
            data.clearAllLinks();
            
         /*
          *  Set new item to calendar for Generator Item
          */
            
            for(i = 0; i < data.ref_field.size(); i++) {
                
                t_item_2 = ((GeneratorRef) data.ref_field.get(i));
               // System.out.println("0"+t_item_2.Generator.getName() + t_item_2.Generator.getStateAsString());
                
                try {
                    if (t_item_2.Generator.getState() == 1) {
                       // System.out.println("1"+t_item_2.Generator.getName() + t_item_2.Generator.getStateAsString() + "delta:"+ delta);
                        t_item_2.Generator.activate(newTime + 0.1);
                    }
                    if (t_item_2.Generator.getState() == 2) {
                       // System.out.println("2"+t_item_2.Generator.getName() + t_item_2.Generator.getStateAsString());
                        t_item_2.Generator.cancel();
                       // System.out.println("2"+t_item_2.Generator.getName() + t_item_2.Generator.getStateAsString());
                        t_item_2.Generator.activate(newTime + 0.1);
                    }
                 //   System.out.println("3" + t_item_2.Generator.getName() + t_item_2.Generator.getStateAsString());
                }
                catch (JSimInvalidParametersException e) {
                    System.out.println("Exception :" + t_item_2.Generator.getStateAsString());
                }
                catch (JSimSecurityException e) {
                    System.out.println("Exception :" + t_item_2.Generator.getStateAsString());
                }
            }
            
        /*
         *  Set new item to calendar for Server Item
         */
           // message("Pocet serveru:" + data.simData.allSimItem.size());
            for(i = 0; i < data.simData.allSimItem.size(); i++) {
                
                t_item_1 = ((SimServerItem) data.simData.allSimItem.get(i));
                //System.out.println("0" + t_item_1.serverName + t_item_1.serverRef.getStateAsString());
                delta = delta * (i + 1);
                try {
                    if (t_item_1.serverRef.getState() == 1) {
                     //   System.out.println("1." + t_item_1.serverName + t_item_1.serverRef.getStateAsString() + "delta:"+ delta);
                        t_item_1.serverRef.activate(newTime + delta);
                    }
                    if (t_item_1.serverRef.getState() == 2) {
                      //  System.out.println("2." + t_item_1.serverName + t_item_1.serverRef.getStateAsString());
                        t_item_1.serverRef.cancel();
                      //  System.out.println("2." + t_item_1.serverName + t_item_1.serverRef.getStateAsString());
                        t_item_1.serverRef.activate(newTime + delta);
                    }
                    if (t_item_1.serverRef.getState() == 3) {
                      //  System.out.println("4." + t_item_1.serverName + t_item_1.serverRef.getStateAsString());
                        t_item_1.serverRef.cancel();
                        t_item_1.serverRef.activate(newTime + delta);
                    }

                    // restore all servers item
                    ((Server) t_item_1.serverRef).restore(t_item_1.serverRef);
                  //  System.out.println("3" + t_item_1.serverName + t_item_1.serverRef.getStateAsString());
                }
                catch (JSimInvalidParametersException e) {
                    System.out.println("Exception :" + t_item_1.serverRef.getStateAsString());
                }
                catch (JSimSecurityException e) {
                    System.out.println("Exception :" + t_item_1.serverRef.getStateAsString());
                }
            }
        }
        else System.out.println("Anti-message comin but rollback operation not needed");
        
        return;
    }
    
    public void deleteAllItemInCalender(JSimProcess ParProcess) {
        try {
            this.deleteEntriesInCalendar(ParProcess, true);
        }
        catch (JSimInvalidParametersException e) {
            message("Exception :" + e);
        }
    }
    
    
    public int getNumberOfUndo(){
        return(numberOfUndo);
    }
    
    public int getNumberOfRollBack(){
        return(numberOfRollBack);
    }
} // class Generator

