/*
 *  A simple generator. Creates new transactions and puts them into a queue.
 *  Look into the documentation for additional information.
 *
 */

import cz.zcu.fav.kiv.jsim.*;
import cz.zcu.fav.kiv.jsim.unsecure.*;

public class NetPoint extends JSimProcess {
    
    private double                  simEndTime = 1000.1;
    private QueueWithServer         queue;
    private QueueWithServer[]       fields_queue;
    private ClientData              data;
    private MessageItem             t_item;
    private double                  wallClock = 0.0;
    private double                  dif = 0.0;
    public int                      inCominCount = 0; // count of incoming messages
    public int                      net_pom = 0;
    public DistributedSimulation    simulation;
    private boolean                 constructorStatus = false;
    //public  UnsecureSimulation      simulation;
    
    public NetPoint(String name, DistributedSimulation sim, QueueWithServer[] q, ClientData ParData) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException {
        super(name, sim);
        fields_queue = q;
        data = ParData;
        simulation = sim;
        constructorStatus = false;
        message("Input field");
    } // constructor
    
    public NetPoint(String name, DistributedSimulation sim, QueueWithServer q, ClientData ParData) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException {
        super(name, sim);
        queue = q;
        data = ParData;
        simulation = sim;
        constructorStatus = true;
        message("Input single queue");
    } // constructor
    
    protected void life() {
        JSimLink        link;
        Transaction     t;
        double          creationTime = 0.0;
        double          newTime = 0.0;
        int             countOfItems = 0;
        boolean         sendAntiMessage = false;
        
        try {
            while (true) {
                
                t_item = data.getFromInputQueue();
                if (t_item.flag == -2){
                    message(getName() + " Last Event Deadlock");
                    hold(simEndTime - myParent.getCurrentTime());
                    simulation.deleteAllItemInCalender(this);
                    this.passivate();
                }
                
                if (t_item.message_1 < myParent.getCurrentTime()){
                    
                    if ((data.last_time == true) && (t_item.flag == -1)){
                        //message(getName() + " Last Time");
                        hold(simEndTime - myParent.getCurrentTime());
                    }
                    else {
                        message("- U N D O - ");
                        message(getName() + ": CT :" + myParent.getCurrentTime()+ ": NextTS :" + t_item.message_1);
                        // zastavit simulaci
                        sendAntiMessage = false;
                        
                        data.WriteInQToFile();
                        data.WriteOutQToFile();
                        
                        data.undo();
                        
                        // zjisti T_stamp(t-1) ktera ma casovou znacku mensi nez T_stamp(t)
                        // nastav vsechny polozky v input queue s casovou znackou vetsi nez T_stamp(t-1) ne not executed
                        countOfItems = data.setNotExecuted(t_item.message_1);
                        message("1." + this.getName() + ": Count of inccorect processed Events in input Queue:" + countOfItems);
                        data.deleteRolledEvents(t_item.message_1);
                        // vymazaz vsechny vysledky vsech komponent simulace od doby >> T_stamp(t-1)
                        countOfItems = data.eraseSimItem(t_item.message_1);
                        message("1." + this.getName() + ": Count of inccorect processed Events in Simulation :" + countOfItems);
                        
                        
                        // musime poslat nejake anti-zpravy?
                        if (data.checkOutGoingMessages(t_item.message_1) == 1) {
                            sendAntiMessage = true;
                        }
                        else {
                            sendAntiMessage = false;
                        }
                        // vymaz vsechni komunikacni buffery
                        data.clearAllLinks();
                        
                        // nastavit cas simulace na na T_stamp
                        simulation.operationUndo(t_item.message_1, data);
                        
                        // posli serveru ze si pripraven na spusteni a cekej na potvrzeni GO_ALL
                        undo2(sendAntiMessage);
                        
                        t = new Transaction(myParent.getCurrentTime());
                        creationTime = data.getCreationTime(myParent.getCurrentTime());
                        
                        if (creationTime < 0.0)
                            message(getName() + " NOT FOUND" + ": CT :" + myParent.getCurrentTime());
                        else {
                            t.setCreationTime(creationTime);
                            link = new JSimLink(t);
                            
                            // contructor without field of queues
                            if (this.constructorStatus == true) {
                                link.into(queue);
                                data.WriteToSimFile(getName() + ": Create CT :" + myParent.getCurrentTime() + ": CTS :" +  creationTime);
                                if (queue.getServer().isIdle()) {
                                    queue.getServer().activate(myParent.getCurrentTime());
                                }
                            }
                            else { // field of queues
                                for (int j = 0; j < 10; j++){
                                    if (((t_item.gate % 10) == j) && (this.fields_queue[j] != null)){
                                        link.into(this.fields_queue[j]);
                                        data.WriteToSimFile(getName() + ": Create CT :" + myParent.getCurrentTime() + ": CTS :" +  creationTime);
                                        if (this.fields_queue[j].getServer().isIdle()) {
                                            this.fields_queue[j].getServer().activate(myParent.getCurrentTime());
                                        }
                                    }
                                    else {
                                        if (((t_item.gate % 10) == j) && (this.fields_queue[j] == null)){
                                            message("Input gate is wrong or input queue is not init !");
                                        }
                                    }
                                }
                            }
                            
                            inCominCount++;
                            //data.WriteToSimFile(getName() + ": Create CT :" + myParent.getCurrentTime() + ": CTS :" +  creationTime);
                        }
                    }
                } // neni undo
                else {
                    data.insertLBTS(t_item.from.toString(), t_item.message_1);
                    hold(t_item.message_1 - myParent.getCurrentTime());
                    t = new Transaction(myParent.getCurrentTime());
                    creationTime = data.getCreationTime(myParent.getCurrentTime());
                    
                    if (creationTime < 0.0)
                        message(getName() + " NOT FOUND" + ": CT :" + myParent.getCurrentTime());
                    else {
                        t.setCreationTime(creationTime);
                        link = new JSimLink(t);
                        // contructor without field of queues
                        if (this.constructorStatus == true) {
                            link.into(queue);
                            data.WriteToSimFile(getName() + ": Create CT :" + myParent.getCurrentTime() + ": CTS :" +  creationTime);
                            if (queue.getServer().isIdle()) {
                                queue.getServer().activate(myParent.getCurrentTime());
                            }
                        }
                        else { // field of queues
                            for (int j = 0; j < 10; j++){
                                if (((t_item.gate % 10) == j) && (this.fields_queue[j] != null)){
                                    //message("gate :" + t_item.gate % 10);
                                    link.into(this.fields_queue[j]);
                                    data.WriteToSimFile(getName() + ": Create CT :" + myParent.getCurrentTime() + ": CTS :" +  creationTime);
                                    if (this.fields_queue[j].getServer().isIdle()) {
                                        this.fields_queue[j].getServer().activate(myParent.getCurrentTime());
                                    }
                                }
                                if (((t_item.gate % 10) == j) && (this.fields_queue[j] == null)){
                                    message("Input gate is wrong or input queue is not init !");
                                }
                            }
                        }
                        inCominCount++;
                    }
                    if ((data.last_time == true) && (t_item.flag == -1)){
                        message("3." + getName() + " Curent time :" + myParent.getCurrentTime()+ ':' + t_item.message_1);
                        hold(simEndTime - myParent.getCurrentTime());
                    }
                }// else undo
            } // while
        } // try
        catch (JSimException e) {
            e.printStackTrace();
            e.printComment(System.err);
        }
    } // life
    
    private void undo2(boolean ParAntiMessageOccur) {
        
        while(data.order_stop_all_ack == false) {
            data.thread_sleep(50);
        }
        
        net_pom++;
        
        if (ParAntiMessageOccur == true) {
            
            data.order_am_all = true;
            while(data.order_am_all_ack == false){
                //System.out.print(".");
                data.thread_sleep(50);
            }
        }
        
        // System.out.println("Go_all");
        data.order_go_all = true;
        
        while(data.order_go_all_ack == false) {
            //System.out.print(".");
            data.thread_sleep(50);
        }
        
        data.order_go_all = false;
        data.order_go_all_exec = false;
        data.order_go_all_ack = false;
        
        data.order_stop_all = false;
        data.order_stop_all_exec = false;
        data.order_stop_all_ack = false;
        
        data.order_am_all = false;
        data.order_am_all_exec = false;
        data.order_am_all_ack = false;
        
        return;
    }
    
    /**
     *  This Mmethod set simulation end time        <BR>
     *
     */
    public void setSimulationEndTime(double ParTime){
        this.simEndTime = ParTime;
    }
} // class Generator

