/*
 *	Copyright (c) 2000 - 2004, University of West Bohemia,
 *	                           Faculty of Applied Sciences,
 *	                           Department of Computer Science and Engineering,
 *	                           DSS Group.
 *	                           Pilsen, Czech Republic.
 *
 *	Please read the licence agreement carefully before using any part of this software.
 */
package cz.zcu.fav.kiv.jsim;

import java.io.PrintStream;

/**
 *	The JSimException class is a general exception used by J-Sim to inform user about an error.
 *	Unlike its subclasses, it is almost never instantiated.
 *	You should never need to create an instance of this class.
 *
 *	@author		<a href="http://home.zcu.cz/~jkacer">Jarda KAER</a>
 *
 *	@version	J-Sim version 0.3.0
 *
 *	@since		J-Sim version 0.0.1
 */
public class JSimException extends Exception
{
	/**
	 *	Specific message for every exception.
	 */
	private String specific;

//--------------------------------------------------------------------------------------------------

	/**
	 *	Creates a new JSimException with a description and a specific information. Descendants will use this
	 *	constructor to pass their own data to the Exception constructor.
	 *
	 *	@param description	Describes the error that occured.
	 *	@param param		Holds detailed information about specific circumstances of this exception,
	 *				for example a method's or a variable's name.
	 */
	public JSimException(String description, String param)
	{
		super(description);
		specific = param;
	} // constructor

	/**
	 *	Creates a new JSimException with the default description and a specific information.
	 *
	 *	@param param	Holds detailed information about specific circumstances of this exception,
	 *			for example a method's or a variable's name.
	 */
	public JSimException(String param)
	{
		this("This is a general J-Sim exception.", param);
	} // constructor

//--------------------------------------------------------------------------------------------------

	/**
	 *	Prints an explanation why this exception was thrown. The explanation is printed out to a print stream.
	 *
	 *	@param ps	The print stream that the explanation is to be printed to.
	 */
	public void printComment(PrintStream ps)
	{
		ps.println("This exception is thrown whenever something suspicious or dangerous happens");
		ps.println("inside your simulation but it is too difficult to determine");
		ps.println("the exact reason of this abnormality.");
		ps.println("Please check up your source code.");
		ps.println();

		if (specific != null)
		{
			ps.println("Additional information: " + getSpecificInfo());
			ps.println();
		} // if
	} // printComment

	/**
	 *	Returns detailed information about specific circumstances of the exception.
	 *
	 *	@return	Detailed information about specific circumstances of the exception.
	 */
	public String getSpecificInfo()
	{
		return specific;
	} // getSpecificInfo

} // class JSimException

