/*
 *	Copyright (c) 2000 - 2004, University of West Bohemia,
 *	                           Faculty of Applied Sciences,
 *	                           Department of Computer Science and Engineering,
 *	                           DSS Group.
 *	                           Pilsen, Czech Republic.
 *
 *	Please read the licence agreement carefully before using any part of this software.
 */
package cz.zcu.fav.kiv.jsim;

import java.io.PrintStream;

/**
 *	The JSimSecurityException is thrown whenever the user tries to execute an action that
 *	is forbidden in the current simulation context.
 *	You should never need to create an instance of this class.
 *
 *	@author		<a href="http://home.zcu.cz/~jkacer">Jarda KAER</a>
 *
 *	@version	J-Sim version 0.3.0
 *
 *	@since		J-Sim version 0.0.2
 */
public class JSimSecurityException extends JSimException
{
	/**
	 *	Creates a new JSimSecurityException with a specific information.
	 *
	 *	@param pars	Holds detailed information about specific circumstances of this exception,
	 *			for example a method's or a varibale's name.
	 */
	public JSimSecurityException(String pars)
	{
		super("J-Sim cannot execute the required action because of a (potential) security violation.", pars);
	} // constructor

	/*
	 *	Method comments are inherited.
	 */
	public void printComment(PrintStream ps)
	{
		ps.println("It is not allowed to execute the action you required because");
		ps.println("the action could demage J-Sim internal structures");
		ps.println("or cause J-Sim to hang or operate improperly.");
		ps.println();

		if (getSpecificInfo() != null)
		{
			ps.println("Additional information: " + getSpecificInfo());
			ps.println();
		} // if
	} // printComment

} // class JSimSecurityException

