/*
 *	Copyright (c) 2000 - 2004, University of West Bohemia,
 *	                           Faculty of Applied Sciences,
 *	                           Department of Computer Science and Engineering,
 *	                           DSS Group.
 *	                           Pilsen, Czech Republic.
 *
 *	Please read the licence agreement carefully before using any part of this software.
 */
package cz.zcu.fav.kiv.jsim.gui;

import cz.zcu.fav.kiv.jsim.*;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 *	The JSimDetailedInfoWindow provides a dialog that shows a table with detailed information
 *	about a process, a queue, or any JSimDisplayable instance.
 *
 *	@author		Pavel DOMECK
 *	@author		<a href="http://home.zcu.cz/~jkacer">Jarda KAER</a>
 *
 *	@version	J-Sim version 0.3.0
 *
 *	@since		J-Sim version 0.2.0
 */
public class JSimDetailedInfoWindow extends JDialog implements Observer
{
	/**
	 *	The owner of this dialog.
	 */
	protected JSimMainWindow myParent;

	/**
	 *	The object whose characteristics will be displayed in this dialog.
	 */
	protected JSimDisplayable objectInfo;

	private JPanel panel;
	private JButton button;
	private JTable table;
	private DefaultTableModel tableModel;
	private JScrollPane tableScrollPane;

//--------------------------------------------------------------------------------------------------

	/**
	 *	The dialog's window adapter. Disposes the dialog of when closed.
	 */
	class MyWindowAdapter extends WindowAdapter
	{
		public void windowClosing(WindowEvent e)
		{
			myParent.openDetailedWindows.remove(objectInfo);
			setVisible(false);
			dispose();
		} //windowClosing
	} // inner class MyWindowAdapter

//--------------------------------------------------------------------------------------------------

	/**
	 *	The dialog's action adapter. Disposes the dialog of when the OK button is pressed.
	 */
	class MyActionAdapter implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			myParent.openDetailedWindows.remove(objectInfo);
			setVisible(false);
			dispose();
		} // actionPerformed
	} // inner class MyActionAdapter

//--------------------------------------------------------------------------------------------------

	/**
	 *	Creates a new dialog containing a table and an OK button.
	 *
	 *	@param parent	The parent window of the dialog, usually the main window.
	 *	@param info	The object to be displayed in the table.
	 */
	public JSimDetailedInfoWindow(JSimMainWindow parent, JSimDisplayable info)
	{
		super(parent, "Details", false);
		myParent = parent;
		objectInfo = info;

		constructorWindowInit();
		constructorSpecificInit();

		pack();

		// Randomize the window's position
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		int x = (int) Math.rint((d.getWidth()-200) * Math.random());
		int y = (int) Math.rint((d.getHeight()-150) * Math.random());
		setLocation(x, y);
	} // constructor

//--------------------------------------------------------------------------------------------------

	/**
	 *	Adds basic components and listeners to the window.
	 */
	protected void constructorWindowInit()
	{
		panel = new JPanel();
		button = new JButton("OK");
		panel.add(button);
		getContentPane().add(panel, BorderLayout.SOUTH);
		button.addActionListener(new MyActionAdapter());
		addWindowListener(new MyWindowAdapter());
	} // constructorWindowInit

	/**
	 *	Creates and adds specific objects to the window, such as a table and its table model.
	 */
	protected void constructorSpecificInit()
	{
		tableModel = new DefaultTableModel();
		String[] columnNames = {"Name", "Value"};
		tableModel.setColumnCount(2);
		tableModel.setColumnIdentifiers(columnNames);

		table = new JTable(tableModel);
		table.setColumnSelectionAllowed(false);
		table.setRowSelectionAllowed(false);
		tableScrollPane = new JScrollPane(table);
		getContentPane().add(tableScrollPane, BorderLayout.CENTER);
		// We must call update to fill the table model
		update(null, null);
		table.setPreferredScrollableViewportSize(new Dimension(200, table.getRowCount() * table.getRowHeight()));
	} // constructorSpecificInit

	/**
	 *	Updates the table model.
	 *	The necessary data for the table model are obtained from the getDetailedInformationArray() method.
	 */
	public void update(Observable o, Object arg)
	{
		Collection collection;
		Iterator it;
		Object obj;
		JSimPair jsp;
		int i = 0;

		collection = objectInfo.getDetailedInformationArray();
		if ((collection != null))
		{
			tableModel.setRowCount(collection.size());

			it = collection.iterator();
			while (it.hasNext())
			{
				obj = it.next();
				if (obj instanceof JSimPair)
				{
					jsp = (JSimPair) obj;
					tableModel.setValueAt(jsp.getName(), i, 0);
					tableModel.setValueAt(jsp.getValue(), i, 1);
				}
				else
					myParent.getSimulation().talk("An object supplied a characteristics description that is not a JSimPair.");
				i++;
			} // while
		} // if
	} // update

} // JSimDetailedInfoWindow

