/*
 *	Copyright (c) 2000 - 2004, University of West Bohemia,
 *	                           Faculty of Applied Sciences,
 *	                           Department of Computer Science and Engineering,
 *	                           DSS Group.
 *	                           Pilsen, Czech Republic.
 *
 *	Please read the licence agreement carefully before using any part of this software.
 */
package cz.zcu.fav.kiv.jsim.gui;

import cz.zcu.fav.kiv.jsim.*;

import java.awt.*;
import javax.swing.*;
import java.util.*;

/**
 *	A JSimMainWindowList is a list whose items are simple descriptions of processes, queues, or other simulation
 *	elements. The elements must implement JSimDisplayable.
 *
 *	@author		Pavel DOMECK
 *	@author		<a href="http://home.zcu.cz/~jkacer">Jarda KAER</a>
 *
 *	@version	J-Sim version 0.3.0
 *
 *	@since		J-Sim version 0.1.2
 */
public class JSimMainWindowList extends JList implements Observer
{
	/**
	 *	The simulation that this list's main window belongs to.
	 */
	private JSimSimulation myParent;

	/**
	 *	Type of the list: contains processes, queues, ...
	 */
	private int listType;

	/**
	 *	A set of objects delivered by the simulation that this list has to display.
	 *	If the set is ordered, the list will also be ordered.
	 */
	private Set objectsToDisplay;

	/**
	 *	An array list of displayed objects. A selected item's position is used as an index to this array list.
	 */
	private ArrayList myInfoTable;

	/**
	 *	The list's list model.
	 */
	private DefaultListModel listModel;

//--------------------------------------------------------------------------------------------------

	/**
	 *	Creates a new JList whose items will be descriptions of processes or queues.
	 *
	 *	@param parent	The simulation object that owns the main window that this list belongs to.
	 *	@param lType	The type of the list: Will it display processes or queues?
	 */
	public JSimMainWindowList(JSimSimulation parent, int listType)
	{
		super();
		try
		{
			myParent = parent;
			this.listType = listType;
			myInfoTable = new ArrayList();
			objectsToDisplay = myParent.getObjectsToBeDisplayed(listType);
			listModel = new DefaultListModel();
			this.setModel(listModel);
			setFont(new Font("SansSerif", Font.PLAIN, 10));
		} // try
		catch (JSimInvalidParametersException e)
		{
			objectsToDisplay = null;
			setModel(null);
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // constructor

//--------------------------------------------------------------------------------------------------

	/**
	 *	Updates the items in the list if the state of displayed objects has changed since
	 *	last call of update().
	 */
	public void update(Observable o, Object arg)
	{
		Iterator it;
		JSimDisplayable jsd;
		String description;

		listModel.clear();
		myInfoTable.clear();

		it = objectsToDisplay.iterator();
		while (it.hasNext())
		{
			jsd = (JSimDisplayable) it.next();
			description = jsd.getObjectListItemDescription();
			if (description == null)
				description = "<Description not available>";
			listModel.addElement(description);
			myInfoTable.add(jsd);
		} // while has next
	} // update

	/**
	 *	Return a table of all items displayed in the list. It is important to provide such an ordered
	 *	list because the main window will access the displayed objects via their indices when a double-click
	 *	occurs.
	 *
	 *	@return	An array list containing all displayed objects in the same order that they are displayed.
	 */
	protected ArrayList getInfoTable()
	{
		return myInfoTable;
	} // getInfoTable

} // class JSimMainWindowList

