/*
 *	Copyright (c) 2000 - 2004, University of West Bohemia,
 *	                           Faculty of Applied Sciences,
 *	                           Department of Computer Science and Engineering,
 *	                           DSS Group.
 *	                           Pilsen, Czech Republic.
 *
 *	Please read the licence agreement carefully before using any part of this software.
 */
package cz.zcu.fav.kiv.jsim;

import java.io.PrintStream;

/**
 *	The JSimSimulationAlreadyTerminatedException is thrown whenever the user attempts to add a new process
 *	into a terminated simulation. Simulation is terminated if there are no more scheduled events or
 *	there are no more processes.
 *	You should never need to create an instance of this class.
 *
 *	@author		<a href="http://home.zcu.cz/~jkacer">Jarda KAER</a>
 *
 *	@version	J-Sim version 0.3.0
 *
 *	@since		J-Sim version 0.1.0
 */
public class JSimSimulationAlreadyTerminatedException extends JSimException
{
	/**
	 *	Creates a new JSimSimulationAlreadyRunningException with a specific information.
	 *
	 *	@param pars	Holds detailed information about specific circumstances of this exception,
	 *			for example a method's or a variable's name.
	 */
	public JSimSimulationAlreadyTerminatedException(String pars)
	{
		super("You cannot add new processes to a terminated simulation.", pars);
	} // constructor

	/*
	 *	Method comments are inherited.
	 */
	public void printComment(PrintStream ps)
	{
		ps.println("If a simulation has already terminated");
		ps.println("it is not allowed to add new processes into it.");
		ps.println("You should reinitialize the simulation or avoid inserting");
		ps.println("new processes at this time.");
		ps.println("Please check up your source code.");
		ps.println();

		if (getSpecificInfo() != null)
		{
			ps.println("Additional information: " + getSpecificInfo());
			ps.println();
		} // if
	} // printComment

} // class JSimSimulationAlreadyTerminatedException

