/*
 *	Copyright (c) 2000 - 2004, University of West Bohemia,
 *	                           Faculty of Applied Sciences,
 *	                           Department of Computer Science and Engineering,
 *	                           DSS Group.
 *	                           Pilsen, Czech Republic.
 *
 *	Please read the licence agreement carefully before using any part of this software.
 */
package cz.zcu.fav.kiv.jsim.unsecure;

import cz.zcu.fav.kiv.jsim.*;

/**
 *	UnsecureHead is an <em>experimental</em> class that you probably will not use at all. It is intended to be used
 *	in distributed simulation where simulation time rollbacks can occur. Therefore, head statistics must also be
 *	reverted sometimes, which is just what this class provides. You had better not use this class at all. If you
 *	do not use the new methods correctly, results of some statistical methods may be completely wrong. However,
 *	the basic functionality should work correctly.
 *
 *	@author		<a href="http://home.zcu.cz/~jkacer">Jarda KAER</a>
 *
 *	@version	J-Sim version 0.3.0
 *
 *	@since		J-Sim version 0.3.0
 */
public class UnsecureHead extends JSimHead
{
	/**
	 *	Creates a new UnsecureHead queue belonging to a simulation and having a name.
	 *	The queue is initially empty.
	 *
	 *	@param name	The name of the queue.
	 *	@param parent	The parent simulation.
	 *
	 *	@exception JSimInvalidParametersException	This exception is thrown out if the parent simulation
	 *							is null.
	 *	@exception JSimTooManyHeadsException		This exception is thrown out if no other queue can be
	 *							added to the simulation specified.
	 */
	public UnsecureHead(String name, JSimSimulation parent) throws JSimInvalidParametersException, JSimTooManyHeadsException
	{
		super(name, parent);
	} // constructor

	/**
	 *	Returns the number of links already removed from the queue.
	 *
	 *	@return	The number of links already removed from the queue.
	 */
	public long getNoOfItemsRemoved()
	{
		return noOfItemsRemoved;
	} // getNoOfItemsRemoved

	/**
	 *	Changes the number of links already removed from the queue. Use with care!
	 *
	 *	@param newValue	The new number of links already removed from the queue.
	 */
	public void setNoOfItemsRemoved(long newValue)
	{
		noOfItemsRemoved = newValue;
	} // setNoOfItemsRemoved

	/**
	 *	Returns the total time spent in the queue by all links already removed from the queue.
	 *
	 *	@return	The total time spent in the queue by all links already removed from the queue.
	 */
	public double getSumTwRemoved()
	{
		return sumTwRemoved;
	} // getSumTwRemoved

	/**
	 *	Changes the total time spent in the queue by all links already removed from the queue. Use with care!
	 *
	 *	@param newValue	The new total time spent in the queue by all links already removed from the queue.
	 */
	public void setSumTWRemoved(double newValue)
	{
		sumTwRemoved = newValue;
	} // setSumTWRemoved

	/**
	 *	Returns the sum of time spent by all links in the queue from its creation to its last change.
	 *
	 *	@return	The sum of time spent by all links in the queue from its creation to its last change.
	 */
	public double getSumLwUntilLastChange()
	{
		return sumLwUntilLastChange;
	} // getSumLwUntilLastChange

	/**
	 *	Changes the sum of time spent by all links in the queue from its creation to its last change.
	 *	Use with care!
	 *
	 *	@param newValue	The new sum of time spent by all links in the queue from its creation to its last change.
	 */
	public void setSumLwUntilLastChange(double newValue)
	{
		sumLwUntilLastChange = newValue;
	} // setSumLwUntilLastChange

} // class UnsecureHead

